#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <cstdio>

#include <QPainter>
#include <QResizeEvent>
#include <QHBoxLayout>

static unsigned int s_Counter = 0;

static const int OVERLAY_HEIGHT = 100;

class QMyWidgetyThing : public QWidget
{
public:
    QMyWidgetyThing(QWidget* parent)
    : QWidget(parent)
    {
        setSizePolicy(QSizePolicy::Fixed, QSizePolicy::MinimumExpanding);
        setMouseTracking(true);
    }

    virtual QSize sizeHint() const
    {
        return QSize(64, 64);
    }

    virtual void paintEvent(QPaintEvent*)
    {
        QPainter painter(this);

        painter.fillRect(rect(), Qt::blue);

        painter.setFont(QFont("Arial", 16));
        QString s;
        s.setNum(s_Counter % 1000);
        painter.setPen(Qt::white);
        painter.drawText(8, 18, s);
    }

    virtual void mouseMoveEvent(QMouseEvent*)
    {
        update();
    }
};
class QMyOverlay : public QWidget
{
public:
    QMyOverlay(QWidget* parent)
    : QWidget(parent)
    {
        setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);

        QHBoxLayout* l = new QHBoxLayout(this);

        for(int i = 0; i < 5; i++) {
            QMyWidgetyThing* widget = new QMyWidgetyThing(this);
            l->addWidget(widget);
        }
    }

    virtual QSize sizeHint() const
    {
        return QSize(0, OVERLAY_HEIGHT);
    }

    virtual void paintEvent(QPaintEvent*)
    {
        QPainter painter(this);

        painter.fillRect(rect(), Qt::gray);
    }
};

class QMyWidget : public QWidget
{
public:
    QMyWidget(QWidget* parent)
    : QWidget(parent),
      m_OverlayWidget(0)
    {
    }

    virtual QSize sizeHint() const
    {
        return QSize(600, 500);
    }

    virtual void paintEvent(QPaintEvent*)
    {
        fprintf(stderr, "repaint %d", s_Counter);
        QPainter painter(this);
        QLinearGradient gradient(0, 0, 0, height());
        gradient.setColorAt(0.0, Qt::red);
        gradient.setColorAt(0.5, Qt::green);
        gradient.setColorAt(1.0, Qt::blue);
        painter.fillRect(rect(), QBrush(gradient));

        painter.setFont(QFont("Arial", 64));
        QString s;
        s.setNum(s_Counter++ % 1000);
        painter.setPen(Qt::black);
        painter.drawText(19, 243, s);
        painter.setPen(Qt::white);
        painter.drawText(16, 240, s);
    }

    void setOverlayWidget(QMyOverlay* overlay)
    {
        m_OverlayWidget = overlay;
		setMask(QRect(0, y() + OVERLAY_HEIGHT, width(), height() - OVERLAY_HEIGHT)); 
    }

    virtual void resizeEvent(QResizeEvent* e)
    {
        if (e && m_OverlayWidget) {
            m_OverlayWidget->resize(e->size().width(), OVERLAY_HEIGHT);
			setMask(QRect(0, y() + OVERLAY_HEIGHT, width(), height() - OVERLAY_HEIGHT)); 
        }
    }

private:
    QMyOverlay*     m_OverlayWidget;
};

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    QMyWidget* myWidget = new QMyWidget(this);
    setCentralWidget(myWidget);

    QMyOverlay* overlay = new QMyOverlay(this);
    overlay->setGeometry(0, 0, width(), OVERLAY_HEIGHT);

    myWidget->setOverlayWidget(overlay);
}

MainWindow::~MainWindow()
{
    delete ui;
}
